@@ -1,4 +1,3 @@
-# $Id: ExecuteArray.pm 15453 2012-10-25 09:05:48Z mjevans $
 package ExecuteArray;
 use Test::More;
 use Data::Dumper;
@@ -225,19 +224,21 @@ sub insert
             }
         };
     }
+    my $ev = $@;
     if ($ref->{error} && $ref->{raise}) {
-        ok($@, 'error in execute_array eval');
-    } else {
-        if ($ref->{requires_mas} && $@) {
-            diag("\nThis test died with $@");
-            diag("It requires multiple active statement support in the driver and I cannot easily determine if your driver supports MAS. Ignoring the rest of this test.");
-            foreach (@tuple_status) {
-                if (ref($_)) {
-                    diag(join(",", @$_));
-                }
+        ok($ev, 'error in execute_array eval');
+    } elsif ($ref->{requires_mas} && $ev) {
+        diag("\nThis test died with $ev");
+        diag("It requires multiple active statement support in the driver and I cannot easily determine if your driver supports MAS. Ignoring the rest of this test.");
+        foreach (@tuple_status) {
+            if (ref($_)) {
+                diag(join(",", @$_));
             }
-            return 'mas';
         }
+        return 'mas';
+    } elsif ($ref->{raise} && $ev) {
+        BAIL_OUT("Totally unexpected error - $ev");
+    } else {
         ok(!$@, 'no error in execute_array eval') or note($@);
     }
     $dbh->commit if $ref->{commit};
