@@ -36,7 +36,7 @@ require 5.004;
    %TestFieldInfo = (
 		     'COL_A' => [SQL_SMALLINT,-5, SQL_TINYINT, SQL_NUMERIC, SQL_DECIMAL, SQL_FLOAT, SQL_REAL, SQL_INTEGER],
 		     'COL_B' => [SQL_VARCHAR, SQL_CHAR, SQL_WVARCHAR, SQL_WCHAR],
-		     'COL_C' => [SQL_LONGVARCHAR, -1, SQL_WLONGVARCHAR],
+		     'COL_C' => [SQL_LONGVARCHAR, -1, SQL_WLONGVARCHAR, SQL_VARCHAR],
 		     'COL_D' => [SQL_TYPE_TIMESTAMP, SQL_TYPE_DATE, SQL_DATE, SQL_TIMESTAMP ],
 		    );
 
@@ -78,14 +78,16 @@ require 5.004;
            $fields .= "$f ";
            # print "-- $fields\n";
            my $row = get_type_for_column($dbh, $f);
-           $fields .= $row->{TYPE_NAME};
-           if ($row->{CREATE_PARAMS}) {
+           $fields .= $row->{TYPE_NAME};    
+      if ($row->{CREATE_PARAMS}) {
                if ($drvname =~ /OdbcFb/i) {
                    # Firebird ODBC driver seems to be badly broken - for
                    # varchars it reports max size of 32765 when it is 4000
                    if ($row->{TYPE_NAME} eq 'VARCHAR') {
                        $fields .= "(4000)";
                    }
+			   } elsif ($drvname =~ /Pg/) {
+				   ## No lengths ever for TEXT!
                } elsif ($drvname =~ /lib.*db2/) {
                    # in DB2 a row cannot be longer than the page size which is usually 32K
                    # but can be as low as 4K
@@ -95,9 +97,17 @@ require 5.004;
                    }
                } elsif (!exists($row->{COLUMN_SIZE})) {
                    # Postgres 9 seems to omit COLUMN_SIZE
-                   $fields .= '(4000)';
-                   note("WARNING Your ODBC driver is broken - DBI's type_info method should return a hashref containing a key of COLUMN_SIZE and we got " .
-                            join(",", sort keys %$row));
+                   # however see discussion at
+                   # http://www.postgresql.org/message-id/5315E336.40603@vmware.com and
+                   # http://www.postgresql.org/message-id/5315E622.2010904@ntlworld.com
+                   # try and use old ODBC 2 PRECISION value
+                   if (exists($row->{PRECISION})) {
+                       $fields .= '(' . $row->{PRECISION} . ')';
+                   } else {
+                       $fields .= '(4000)';
+                       note("WARNING Your ODBC driver is broken - DBI's type_info method should return a hashref containing a key of COLUMN_SIZE and we got " .
+                                join(",", sort keys %$row));
+                   }
                } else {
                    $fields .= "($row->{COLUMN_SIZE})" if ($row->{CREATE_PARAMS} =~ /LENGTH/i);
                    $fields .= "($row->{COLUMN_SIZE},0)" if ($row->{CREATE_PARAMS} =~ /PRECISION,SCALE/i);
@@ -179,6 +189,9 @@ require 5.004;
 			$dbh->quote($_->[1]),
 			$dbh->quote($_->[2]),
 			$_->[isDateType($row->{DATA_TYPE}) ? 3 : 4]). ")";
+	 if ('Pg' eq $dbh->{Driver}{Name}) {
+		 $sql =~ s/{(?:ts|d) (.+?)}/$1/g;
+	 }
         #diag($sql);
 	 if (!$dbh->do($sql)) {
            diag($dbh->errstr);
